#!/bin/bash

KVER=$1
KDIR=$2

if [ -z "$KVER" ] || [ ! -d "$KDIR" ]; then
	echo "Usage: $0 <kernel_ver> <kernel_dir>"
	exit 1
fi

OFA_DIR=$(echo "$PWD/mlnx-ofed-kernel-"*)

# Build mlnx-ofed-kernel
(cd mlnx-ofed-kernel-* && rm -rf source) || true
(cd mlnx-ofed-kernel-* && mkdir -p source) || exit 2
(cd mlnx-ofed-kernel-* && ls -1 | grep -v source | xargs -iELEM cp ELEM -r source) || exit 2
(cd mlnx-ofed-kernel-* && find compat -type f -exec touch -t 200012201010 '{}' \;) || true
(cd mlnx-ofed-kernel-* && ./configure --kernel-version="$KVER" --kernel-sources="$KDIR" --with-core-mod \
					--with-user_mad-mod --with-user_access-mod --with-addr_trans-mod \
					--with-mlxfw-mod --with-mlx5-mod --with-mlx5-ipsec \
					--with-ipoib-mod) || exit 2
(cd mlnx-ofed-kernel-* && make distclean) || true
(cd mlnx-ofed-kernel-* && make -j$(nproc)) || exit 2

# Build isert
(cd isert-* && make -j$(nproc) KVER="$KVER" K_BUILD="$KDIR" OFA_DIR="$OFA_DIR") || exit 2

# Build iser
(cd iser-* && make -j$(nproc) KVER="$KVER" K_BUILD="$KDIR" OFA_DIR="$OFA_DIR") || exit 2

# Build knem
(cd knem-* && find . -type f -exec touch -t 200012201010 '{}' \;) || true
(cd knem-* && ./autogen.sh) || exit 2
(cd knem-* && ./configure --with-linux-release="$KVER" --with-linux="$KDIR" --with-linux-build="$KDIR") || exit 2
(cd knem-* && make -j$(nproc)) || exit 2

# Build mlnx-nfsrdma
(cd mlnx-nfsrdma-* && make clean) || true
(cd mlnx-nfsrdma-* && make -j$(nproc) KVER="$KVER" K_BUILD="$KDIR" OFA_DIR="$OFA_DIR") || exit 2

# Build mlnx-nvme
(cd mlnx-nvme-* && make clean) || true
(cd mlnx-nvme-* && make -j$(nproc) KVER="$KVER" K_BUILD="$KDIR" CONFIG_NVME_HOST_WITHOUT_FC= OFA_DIR="$OFA_DIR") || exit 2

# Build srp
(cd srp-* && make clean) || true
(cd srp-* && make -j$(nproc) KVER="$KVER" K_BUILD="$KDIR" OFA_DIR="$OFA_DIR") || exit 2

# Build kernel-mft
(cd kernel-mft-* && make clean) || true
(cd kernel-mft-* && make -j$(nproc) all KPVER="$KVER" KSRC="$KDIR") || exit 2

